# This file was auto-generated by Fern from our API Definition.

import typing

Status = typing.Union[typing.Literal["deploying", "running", "paused", "terminated", "error", "warm_pool"], typing.Any]
