# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import datetime as dt
from .get_instance_response_instance_type import GetInstanceResponseInstanceType
from .status import Status
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class GetInstanceResponse(UniversalBaseModel):
    id: str
    launch_time: dt.datetime
    instance_type: GetInstanceResponseInstanceType
    status: Status
    resolution: typing.Optional[typing.List[int]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
